% function [V,l,M]=DLRabbit(level,meas, plotmth, toplot, writeme)  
% ------------------------------------------
% level is the level of the approximation 
% meas is either 'e' for equilibrium or 'c' for conformal 
% plotmth is a character 
%      -'r'- plots just the ray information (2 to a page) 
%      -'c'- plots ray, support and inner circle information (4 to a page) 
%      -'s'- plots ray and support information (4 to a page) (default)
%      -'u'- plots the ray version and the unmatched function
%      -'n'- none turns off ploting
%      -'b'- makes the color fun 3-D plots (not writen out) 
%      if longer string specified only first character will be used  
% toplot is a vector specifying the functions to be plotted 
% writeme (1, or 0) tells the function wether to write out the eigenvalues (if
%     toplot is specified default is 0 else default is 1) 

% Example: DLBunnyAv(6,'c', 's', [1:10,39:40], 1 ); will compute the spectrum for
% the level 6 approximation to the laplacian with conformal measure and
% plot the ray parameterization and support eigenfunctions 1-10 and 39 and
% 40, it will the save the all the eigenvalues that the plots of the
% specified functions. 

% Uses LapBunny.m to set up points, exterior rays and weights then uses this
% information to form a discrete estimate for the laplacian (using the
% p-invarient energy) and evaluate it's eigenvalues and eigenfunctions.

function [V,l,M]=DLRabbit(level,meas, plotmth, toplot, writeme)  
% Taryn Flock Summer 2008

%% Input Processing %% 
% -----------------------------------------------------------------
 if nargin<2 
    error('I need more info! please specify level, and measure')
elseif nargin==2
    plotmth='s';
    toplot=1:10; 
    writeme=0; 
elseif nargin==3; 
    maxfun=min(45,2^level); 
    toplot=1:maxfun;
    writeme=1;
elseif nargin==4;
    writeme=0;
end
r=2^(1/3);
% c=-.122+.745*sqrt(-1);


%% Generate set up info
%---------------------------------
[rays1,vals1,weights1,matches1]=LapBunny(level-2);
numpoints1=length(rays1);
clear vals1 weights1 matches1 
[rays2,vals2,weights2,matches2]=LapBunny(level-1);
numpoints2=length(rays2);
clear vals2 weights2 matches2 
[rays3,vals3,weights3,matches3]=LapBunny(level);
numpoints3=length(rays3);

if (meas=='e') 
    weights3=ones(1,numpoints3)/numpoints3;
elseif (meas~='c') 
    error('Please specify a valid measure, either c, conformal, or e, equalibrium')
end 
  

%% Compute Orginal Energy Matrices 
%-------------------------------------------------------
% Calculate the differences btw ray angles
Drays1=[rays1,0]-[rays1(numpoints1)-7*2^(level-2),rays1];
Drays1=Drays1(1:numpoints1);

clear rays1

% Calculate Energy Matrix
EA=diag(-1./Drays1,0)+diag(1./Drays1(2:numpoints1),1);
EA(numpoints1,1)=1/Drays1(1);
EA=14*EA; 
EB=diag(-1*ones(1,numpoints1),0)+diag(ones(1,numpoints1-1),-1);
EB(1,numpoints1)=1;
E1=r^(level-2)*EA*EB; 

clear EA EB Drays1

% Calculate the differences btw ray angles
Drays2=[rays2,0]-[rays2(numpoints2)-7*2^(level-1),rays2];
Drays2=Drays2(1:numpoints2);
% Calculate Energy Matrix
EA=diag(-1./Drays2,0)+diag(1./Drays2(2:numpoints2),1);
EA(numpoints2,1)=1/Drays2(1);
EA=14*EA; 
EB=diag(-1*ones(1,numpoints2),0)+diag(ones(1,numpoints2-1),-1);
EB(1,numpoints2)=1;
E2=r^(level-1)*EA*EB; 

clear EA EB Drays2

% Calculate the differences btw ray angles
Drays3=[rays3,0]-[rays3(numpoints3)-7*2^(level),rays3];
Drays3=Drays3(1:numpoints3);
% Calculate Energy Matrix
EA=diag(-1./Drays3,0)+diag(1./Drays3(2:numpoints3),1);
EA(numpoints3,1)=1/Drays3(1);
EA=14*EA; 
EB=diag(-1*ones(1,numpoints3),0)+diag(ones(1,numpoints3-1),-1);
EB(1,numpoints3)=1;
E3=r^level*EA*EB;

clear EA EB Drays3
    
%% Compute Average Energy 
% ------------------------------------------------------------------------
% Up E1 
EuppedA=zeros(numpoints2,numpoints1);
EuppedA(~mod(rays2,2),:)=E1; 
clear E1
Eupped1=zeros(numpoints2,numpoints2);
Eupped1(:,~mod(rays2,2))=EuppedA; 

EuppedA=zeros(numpoints3,numpoints2);
EuppedA(~mod(rays3,2),:)=Eupped1; 
Eupped1=zeros(numpoints3,numpoints3);
Eupped1(:,~mod(rays3,2))=EuppedA; 

% Up E2
EuppedA=zeros(numpoints3,numpoints2);
EuppedA(~mod(rays3,2),:)=E2; 
Eupped2=zeros(numpoints3,numpoints3);
Eupped2(:,~mod(rays3,2))=EuppedA; 

E=Eupped1+Eupped2+E3;

clear EuppedA Eupped1 Eupped2 E1 E2 E3 rays2

%% Compute Laplacian 
% Make Matches Map
numpoints=numpoints3;

M =zeros(numpoints,numpoints/3);
for i=1:numpoints/3 
      M(matches3(i,:),i)=ones (1,3);
end       

E=M'*E*M; 

% Calculate 'Integral' wrt measure matrix
G=M'*diag(weights3)*M;

% Laplacian Matrix
L=inv(G)*E;

% Calculate and order eigenvalues
[V,D]=eig(L);

[l,I]=sort(nonzeros(D));

V=V(:,I);

V=CleanVR(V,M,level);

%% Process Data 
% -----------------------------
% Write out data 
if writeme==1; 
    filename=strcat('Bunny',meas,'l',num2str(level),'.out');
    fid=fopen(filename,'w');
    if (fid==-1); error('Bad Filename'); end
    fprintf(fid,'% f\n', l);
    fclose(fid);
end 

% Make Pretty Pictures
MaxPics=length(toplot); 
if plotmth=='c' 
    [incirc,incircp]=InCircleR(level); 
    incircp=logical(incircp);
    for j=1:ceil(MaxPics/4); 
        figure;
        for i=1:min(4,MaxPics-4*(j-1))
            eignum = toplot(i+4*(j-1));
            subplot(4,3,3*i-2);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays3/(max(rays3)+1),eigfun,'LineWidth',1);
            title(strcat('Num',num2str(eignum)));
            axis square
            axis tight
            
            subplot(4,3,3*i-1);
            % Look at sign of sufficiently nonzero points 
            goodpt=abs(eigfun)>2*(max(abs(V(:,1)))-min(abs(V(:,1))));
            goodpt=goodpt&(abs(eigfun)>(max(abs(eigfun))/100));
            signpt=sign(eigfun(goodpt));
            scatter(real(vals3(goodpt)),imag(vals3(goodpt)),5*abs(signpt),signpt, 'filled')
            title(strcat('Eigval',num2str(l(eignum))))
            %cm = [.2 .2 .2; .5 .5 .5];
            colormap(winter);
            axis([-2 2 -1.5 1.5 ]);
            axis square 
            
            subplot(4,3,3*i);
            % Plot Inner Circle 
            eigfunc=eigfun(incircp); % thin out matches 
            plot((1:length(eigfunc))/(length(eigfunc)+1),eigfunc, 'LineWidth',1);
            axis square
            axis tight
            if sum(goodpt(incirc))==0
                ylim([-abs(max(V(:,2))),abs(max(V(:,2)))])
            end
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
        if writeme==1 
            filename = strcat('R',num2str(level),meas,'Pic',num2str(j),'.png'); 
            print('-dpng', filename)
            close
        end 
    end
elseif plotmth=='s' 
    for j=1:ceil(MaxPics/2)
        figure;
        for i=1:min(2,MaxPics-2*(j-1))
            eignum = toplot(i+2*(j-1));
            subplot(2,2,2*i-1);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays3,eigfun, 'LineWidth',1);
            title(strcat('Num',num2str(eignum),'NVal',num2str(l(eignum)/l(2))))
            axis square
            axis tight
            
            subplot(2,2,2*i);
            % Look at sign of sufficiently nonzero points 
            if (level>4) 
                goodpt=abs(eigfun)>10^-(level-3);
            else 
                goodpt=abs(eigfun)>10^-(level-1);
            end
            signpt=sign(eigfun(goodpt));
            scatter(real(vals3(goodpt)),imag(vals3(goodpt)),5*abs(signpt),signpt, 'filled')
            title(strcat('Num',num2str(eignum),'NVal',num2str(l(eignum)/l(2))))
            cm = [0 1 .37; .049 0 1];
            colormap(cm);
            axis([-2 2 -1.5 1.5 ]);
            axis square     
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
    if writeme==1 
        filename = strcat('R',num2str(level),meas,'Pic',num2str(j),'.png'); 
        print('-dpng', filename)
        close
    end
    end
elseif plotmth=='u' 
    for j=1:ceil(MaxPics/2)
        figure;
        for i=1:min(2,MaxPics-2*(j-1))
            eignum = toplot(i+2*(j-1));
            subplot(2,2,2*i-1);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays3,eigfun,':*', 'LineWidth',1);
            title(strcat('Num',num2str(eignum),'NVal',num2str(l(eignum)/l(2))))
            axis square
            axis tight
            
            subplot(2,2,2*i);
            % Plot Unmatched points
            plot(V(:,eignum), ':*', 'LineWidth',1);
            title(strcat('Num',num2str(eignum),'NVal',num2str(l(eignum)/l(2))))
            axis square
            axis tight     
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
    if writeme==1 
        filename = strcat('R',num2str(level),meas,'Pic',num2str(j),'.png'); 
        print('-dpng', filename)
        close
    end
    end
elseif plotmth=='r' 
      for j=1:ceil(MaxPics/3)
        figure;
        for i=1:min(3,MaxPics-3*(j-1))
            eignum = toplot(i+3*(j-1));
            subplot(3,1,i);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays3,eigfun,'k', 'LineWidth',1);
            title(strcat('Num',num2str(eignum),' EigVal ',num2str(l(eignum))))
            axis tight
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
        if writeme==1 
            filename = strcat('R',num2str(level),meas,'Pic',num2str(j),'.png'); 
            print('-dpng', filename)
            close
        end
      end
elseif plotmth=='b'
  for i=toplot
      figure;
      scatter3(real(vals3),imag(vals3),M*V(:,i),15,M*V(:,i),'filled');
      axis([-2 2 -1.5 1.5 min(V(:,i)), max(V(:,i))]) 
  end
 elseif plotmth~='n' 
    fprintf('Invalid plot method please specify r,c,s or n\r')
end

% if plotme==1 
% MaxPics=min(128,length(l));
% npp= 2; % number of functions per page
% for j=1:ceil(MaxPics/npp)
%     figure;
%     for i=1:min(npp,MaxPics-npp*(j-1))
%         subplot(npp,2,2*i-1);
%         plot(rays3,M*V(:,i+npp*(j-1)))
%         title(strcat('NormedEigval',num2str(l(i+npp*(j-1))/l(2))))
%         axis square
%         axis tight
%         
%         subplot(npp,2,2*i);
%         % Look at sign of sufficiently nonzero points 
%         signpt=M*V(:,i+npp*(j-1));
%         goodpt=abs(signpt)>10^-(level-3);
%         signpt=sign(signpt(goodpt));
%         scatter(real(vals3(goodpt)),imag(vals3(goodpt)),4*abs(signpt),signpt,'filled');
%         title(strcat('NormedEigval',num2str(l(i+npp*(j-1))/l(2))))
%         cm = [0 1 .37; .049 0 1];
%         colormap(cm);
%         axis([-2 2 -1.5 1.5 ]);
%         axis square 
%     
%     end
%     set(gcf,'PaperPositionMode','auto');
%     set(gcf,'Position',[1,1,560,700]);
%     filename = strcat('R',num2str(level),meas,'Pic',num2str(j),'.png'); 
%     print('-dpng', filename)
%     close 
% end
% 

